import {
  Component,
  ViewChild,
  OnInit,
  ChangeDetectorRef,
  ElementRef
} from '@angular/core';
import { COMMA, ENTER } from '@angular/cdk/keycodes';
import { PerfectScrollbarDirective } from 'ngx-perfect-scrollbar';
import { CdkVirtualScrollViewport } from '@angular/cdk/scrolling';
import { MatTableDataSource } from '@angular/material/table';
import { SelectionModel } from '@angular/cdk/collections';
import { MatSort } from '@angular/material/sort';
import { FormControl } from '@angular/forms';
import { Observable } from 'rxjs';
import { startWith, map } from 'rxjs/operators';
import {
  MatAutocomplete,
  MatAutocompleteSelectedEvent
} from '@angular/material/autocomplete';
import { MatChipInputEvent } from '@angular/material/chips';

export interface PeriodicElement {
  name: string;
  position: number;
  weight: string;
}

export interface UserInfo {
  /** 사용자SEQ */
  seq: number;
  /** 사용자명 */
  name: string;
}

const ELEMENT_DATA: PeriodicElement[] = [
  { position: 1, name: 'LGCNS', weight: '' },
  { position: 2, name: 'LGCNS', weight: '' },
  { position: 3, name: 'LGCNS', weight: '' },
  { position: 4, name: 'LGCNS', weight: '' },
  { position: 5, name: 'LGCNS', weight: '' },
  { position: 6, name: 'LGCNS', weight: '' }
];

@Component({
  selector: 'app-organization-detail-table',
  templateUrl: './detail-table.component.html',
  styleUrls: ['./detail-table.component.scss']
})
export class DetailTableComponent implements OnInit {
  displayedColumns2: string[] = ['position', 'name', 'weight', 'select'];
  displayedColumns: string[] = [
    'profileImage',
    'profileInfo',
    'company_hpNumber',
    'responsibilities_email',
    'checkable'
  ];

  selectedUserList1: UserInfo[] = [
    { seq: 1, name: 'Helium' },
    { seq: 2, name: 'Hydrogen' },
    { seq: 3, name: 'Lithium' },
    { seq: 4, name: 'Beryllium' },
    { seq: 5, name: 'Carbon' },
    { seq: 6, name: 'Boron' }
  ];

  selectedUserList: UserInfo[] = [
    { seq: 1, name: 'Helium' },
    { seq: 2, name: 'Hydrogen' },
    { seq: 3, name: 'Lithium' },
    { seq: 4, name: 'Beryllium' },
    { seq: 5, name: 'Carbon' },
    { seq: 6, name: 'Boron' }
  ];

  searchItemList: any[] = [{ itemName: '전체' }, { itemName: '부서' }];

  visible = true;
  selectable = true;
  removable = true;
  separatorKeysCodes: number[] = [ENTER, COMMA];
  fruitCtrl = new FormControl();
  filteredFruits: Observable<string[]>;
  fruits: string[] = ['Lemon'];
  allFruits: string[] = ['Apple', 'Lemon', 'Lime', 'Orange', 'Strawberry'];

  @ViewChild(MatSort)
  sort: MatSort;

  @ViewChild('fruitInput') fruitInput: ElementRef<HTMLInputElement>;
  @ViewChild('auto') matAutocomplete: MatAutocomplete;

  dataSource: MatTableDataSource<PeriodicElement>;
  selection: SelectionModel<PeriodicElement>;

  constructor(private changeDetectorRef: ChangeDetectorRef) {
    this.dataSource = new MatTableDataSource<PeriodicElement>(ELEMENT_DATA);
    this.selection = new SelectionModel<PeriodicElement>(true, []);
    this.filteredFruits = this.fruitCtrl.valueChanges.pipe(
      startWith(null),
      map((fruit: string | null) =>
        fruit ? this._filter(fruit) : this.allFruits.slice()
      )
    );
  }

  ngOnInit() {
    this.dataSource.sort = this.sort;
    this.dataSource.filterPredicate = (
      data: PeriodicElement,
      filterJson: string
    ) => {
      const matchFilter = [];
      const filters = JSON.parse(filterJson);

      filters.forEach((filter) => {
        const val = data[filter.id] === null ? '' : data[filter.id];
        matchFilter.push(
          val.toLowerCase().includes(filter.value.toLowerCase())
        );
      });
      return matchFilter.every(Boolean);
    };
  }

  /** Whether the number of selected elements matches the total number of rows. */
  isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.dataSource.data.length;
    return numSelected === numRows;
  }

  /** Selects all rows if they are not all selected; otherwise clear selection. */
  masterToggle() {
    this.isAllSelected()
      ? this.selection.clear()
      : this.dataSource.data.forEach((row) => this.selection.select(row));
  }

  /** The label for the checkbox on the passed row */
  checkboxLabel(row?: PeriodicElement): string {
    if (!row) {
      return `${this.isAllSelected() ? 'select' : 'deselect'} all`;
    }
    return `${this.selection.isSelected(row) ? 'deselect' : 'select'} row ${
      row.position + 1
    }`;
  }

  onClickDeleteUser(userInfo: UserInfo) {
    this.selectedUserList = this.selectedUserList.filter(
      (item) => item.seq !== userInfo.seq
    );
    this.changeDetectorRef.detectChanges();
  }

  add(event: MatChipInputEvent): void {
    const input = event.input;
    const value = event.value;

    // Add our fruit
    if ((value || '').trim()) {
      this.fruits.push(value.trim());
    }

    // Reset the input value
    if (input) {
      input.value = '';
    }

    this.fruitCtrl.setValue(null);
  }

  remove(fruit: string): void {
    const index = this.fruits.indexOf(fruit);

    if (index >= 0) {
      this.fruits.splice(index, 1);
    }
  }

  selected(event: MatAutocompleteSelectedEvent): void {
    this.fruits.push(event.option.viewValue);
    this.fruitInput.nativeElement.value = '';
    this.fruitCtrl.setValue(null);
  }

  private _filter(value: string): string[] {
    const filterValue = value.toLowerCase();

    return this.allFruits.filter(
      (fruit) => fruit.toLowerCase().indexOf(filterValue) === 0
    );
  }
}
